﻿using System.Collections.Generic;
using System.Linq;
using System.IO;

namespace FBX
{
    public class Reader
    {
        public List<FBXNode> Nodes = new List<FBXNode>();
        private TextReader _tr;


        private readonly string _path;


        public Reader(string path)
        {
            _path = path;

        }
        public void Open()
        {
            _tr = new StreamReader(_path);

            ParseElement(Nodes);

            // close the stream
            _tr.Close();
        }

        private void ParseElement(List<FBXNode> nodes)
        {
            //parse all lines in element
            while (ParseLine(_tr.ReadLine(), nodes))
            { }
        }

        private bool ParseLine(string line, List<FBXNode> nodes)
        {
        //sprawdz czy nie koneic pliku
            if (line == null)
                return false;

        //omijamy wiodące białe znaki
            line = OmitWhiteSpace(line);
            if (line == null)
                return true;

        //sprawdzamy czy jest komentarz
            if (line[0] == ';')
                return true;

        //sprawdzamy czy nei koniec elementu
            if (line[0] == '}')
                return false;

        //odczytujemy token
            int i = 0;
            while (line[i] != ':')
                i++;
            string token = line.Substring(0, i);

        //odczytujemy liste atrybutów
            line = line.Remove(0, i+1); //+1, bo omijamy jeszcze ':'

            //sprawdzić czy ostatni ma { na końcu
            int position = line.LastIndexOf('{');
            bool isChild = false;
            if (position >= 0)
            {
                //usuń z lini '{' i do końca lini
                line = line.Remove(position);
                isChild = true;
            }
            else
            {
                //sprawdzenie czy następna linijka zaczyna się od ',' i dołączenie jej
                while(true)
                {
                    char c = (char)_tr.Peek();
                    if (c == ' ')
                        _tr.Read();
                    else if ((c == ',') || (line[line.Length-1] == ',')) //warunek drugi sprawdza czy linia nie kanczy sie przecinkiem
                        line += _tr.ReadLine();
                    else
                        break;
                }
            }

            //usunać Whitespace
            line = line.Trim();

            string[] items = line.Split(',');

            nodes.Add(FBXNodeFactory.Get(token, items[0]));

            //nodes.Add(new FBXNode(token));

        //Zapisać StringNody
            foreach (string s in items)
            {
                //omijać puste ciągi, w sumie tylko ostatni może być pusty
                string attr = OmitWhiteSpace(s);
                if(attr != null)
                    nodes.Last().Attributes.Add(new StringNode(attr));
            }

        //Zapisać FBXNody 
            if(isChild)
            {
                //rozpocznij parsować następny element po otwartej klamrze.
                ParseElement(nodes.Last().Childs);
            }

            return true;
        }

        private string OmitWhiteSpace(string line)
        {
            int i = -1;
            do
            {
                i++;
                if (i == line.Length)
                    return null;
            }
            while (line[i] == ' ');
            return line.Remove(0, i);
        }

        public NodeTakes Takes
        {
            get { return (NodeTakes)Nodes.Find(a => a.Name == "Takes"); }
        }

        public NodeObjects Objects
        {
            get { return (NodeObjects)Nodes.Find(a => a.Name == "Objects"); }
        }
    }
}
