﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

using System.Runtime.InteropServices;

namespace Rotplucs
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public class _3DxConnection
    {
        public static _3DxMouse._3DxMouse  my3DxMouse;

        private Vector3 tv = new Vector3();
        private Vector3 rv = new Vector3();
        public bool Reset = false;

        delegate IntPtr WndProc(IntPtr hWnd, uint msg, IntPtr wParam, IntPtr lParam);
        private static readonly WndProc StaticWndProcDelegate = HookProc;

        static IntPtr prevWndProc;
        const int GWL_WNDPROC = -4;

        [DllImport("user32.dll", CharSet = CharSet.Unicode)]
        static extern int SetWindowLong(IntPtr hWnd, int nIndex, WndProc newProc);
        [DllImport("user32.dll", CharSet = CharSet.Unicode)]
        static extern IntPtr CallWindowProc(IntPtr lpPrevWndFunc, IntPtr hWnd, uint Msg, IntPtr wParam, IntPtr lParam);

        public bool Start3Dx(IntPtr intPtr)
        {
            // Connect to Raw Input & find devices
            my3DxMouse = new _3DxMouse._3DxMouse(intPtr);
            int NumberOf3DxMice = my3DxMouse.EnumerateDevices();

            if (NumberOf3DxMice == 0)
                return false;

            prevWndProc = (IntPtr)SetWindowLong(intPtr, GWL_WNDPROC, StaticWndProcDelegate);

            // Setup event handlers to be called when something happens
            my3DxMouse.MotionEvent += new _3DxMouse._3DxMouse.MotionEventHandler(MotionEvent);
            my3DxMouse.ButtonEvent += new _3DxMouse._3DxMouse.ButtonEventHandler(ButtonEvent);
            return true;
        }

        /// <summary>
        /// Translations and Rotations arrive separately (T first, then R), but some firmware deliver both together
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MotionEvent(object sender, _3DxMouse._3DxMouse.MotionEventArgs e)
        {
            if (e.TranslationVector != null)
            {
                tv.X =  e.TranslationVector.X;
                tv.Y =  -e.TranslationVector.Y;
                tv.Z =  -e.TranslationVector.Z;
            }

            if (e.RotationVector != null)
            {
                rv.X =  e.RotationVector.X;
                rv.Y = -e.RotationVector.Y;
                rv.Z = -e.RotationVector.Z;
            }
        }

        private void ButtonEvent(object sender, _3DxMouse._3DxMouse.ButtonEventArgs e)
        {
            // Show the buttons that are pressed
            //this.ButtonsLabel.Content = e.ButtonMask.Pressed.ToString("X");
            Reset = false;

            if(e.ButtonMask.Mask == 2)
                Reset = true;

        }

        public static IntPtr  HookProc(IntPtr hwnd, uint msg, IntPtr wParam, IntPtr lParam)
        {
            IntPtr returnCode = CallWindowProc(prevWndProc, hwnd, msg, wParam, lParam);

            if (my3DxMouse != null)
            {
                my3DxMouse.ProcessMessage(hwnd, msg, wParam, lParam);
            }

            return returnCode;
        }

        internal Vector3 GetRotationOffset()
        {
            Vector3 offset = new Vector3();
            offset.Y = Pow2(rv.X / 1000);
            offset.X = Pow2(rv.Y / 1000);
            offset.Z = Pow2(rv.Z / 1000);
            return offset;
        }

        internal Vector3 GetPositionOffset()
        {
            Vector3 offset = new Vector3();
            offset.X = Pow2(tv.X / 40);
            offset.Y = Pow2(tv.Y / 40);
            offset.Z = Pow2(tv.Z / 40);
            return offset;
        }

        private static float Pow2(float p)
        {
            return Math.Sign(p) * p * p;
        }
    }
}
