﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using Microsoft.Xna.Framework;



using length = System.Single;
using angle = System.Single;




namespace Rotplucs
{
    public class _MicroScribe
    {

#region const definitions

        const string ARM_MESSAGE	= "MscribeUpdate";

        const int ARM_SUCCESS           = 0;
        const int ARM_ALREADY_USED      = -1;
        const int ARM_NOT_ACCESSED      = -2;
        const int ARM_CANT_CONNECT	    = -3;
        const int ARM_INVALID_PARENT    = -4;
        const int ARM_NOT_CONNECTED     = -5;
        const int ARM_BAD_PORTBAUD	    = -6;

        const int ARM_3DOF     = 0x0001;
        const int ARM_6DOF     = 0x0002;
        const int ARM_3JOINT   = 0x0004;
        const int ARM_6JOINT   = 0x0008;
        const int ARM_FULL     = 0x0010;

        const int ARM_INCHES   = 1;
        const int ARM_MM       = 2;

        const int ARM_DEGREES  = 1;
        const int ARM_RADIANS  = 2;


        /* # of bytes in HCI parameter block, plus extra room */
        const int PARAM_BLOCK_SIZE = 40;
        const int EXT_PARAM_BLOCK_SIZE = 10;
        


#endregion const definitions


#region structs & enums


        /* General 3D spatial coordinate data type */
        public class length_3D
        {
            length  x;
	        length  y;
	        length  z;

            public length_3D(length X, length Y, length Z)
            {
                x = X;
                y = Y;
                z = Z;
            }
        }

        /* General 3D angular coordinate data type */
        public class angle_3D
        {
            angle  x;
	        angle  y;
	        angle  z;

            public angle_3D(angle X, angle Y, angle Z)
            {
                x = X;
                y = Y;
                z = Z;
            }
        }

#endregion


        static uint armMsgVal;

        private IntPtr intPtr;
        public bool isConnected = false;

        public _MicroScribe(IntPtr intPtr)
        {
            //ArmStart(NULL); pointer na okno może być nullem
            if (ArmStart(intPtr) != ARM_SUCCESS)
            {
                isConnected = false;
			}
            else
            {
                //try all available ports - 0 and baud rates - 0L
                if (ArmConnect(0, 0) == ARM_SUCCESS)
                {
                    isConnected = true;
                    ArmSetBckgUpdate(ARM_3DOF);

                    armMsgVal = RegisterWindowMessage(ARM_MESSAGE);
                }
                else
                    isConnected = false;
			}
        }

        


        #region DllImports

       [DllImport("User32.dll")]
        extern static uint RegisterWindowMessage(string msg_type);


        [DllImport("armdll32.dll")]
        extern static int ArmStart(IntPtr parent);

       [DllImport("armdll32.dll")]
        extern static void ArmEnd();

        [DllImport("armdll32.dll")]
        extern static int ArmConnect(int port, int baud);

        [DllImport("armdll32.dll")]
        extern static void ArmDisconnect();

        [DllImport("armdll32.dll")]
        extern static int ArmReconnect();

        [DllImport("armdll32.dll")]
        extern static int ArmSetBckgUpdate(int type);

        [DllImport("armdll32.dll")]
        extern static int ArmSetUpdate(int type);


        [DllImport("armdll32.dll")]
        extern static uint  ArmGetArmRec();


        [DllImport("armdll32.dll")]
        extern static void ArmSetLengthUnits(int type);

        [DllImport("armdll32.dll")]
        extern static void ArmSetAngleUnits(int type);



        [DllImport("armdll32.dll")]
        private static extern int ArmGetTipPosition(ref Vector3 tipPosition);

        [DllImport("armdll32.dll")]
        private static extern int ArmGetTipOrientation(ref Vector3 tipOri);
		
		[DllImport("armdll32.dll")]
        private static extern int ArmGetTipOrientationUnitVector(ref Vector3 tipOri);

        [DllImport("armdll32.dll")]
        private static extern int ArmGetButtonsState(ref uint buts);

#endregion DllImports
        

        internal Vector3 GetTipPosition()
        {
            //length_3D tipPosition;
            Vector3 p = new Vector3();
            int returnValue = ArmGetTipPosition(ref p);

            if (returnValue == ARM_SUCCESS)
                return new Vector3(p.X, p.Y, p.Z);
            else
            {
                return new Vector3(0,0,0);
            }
        }

        internal Vector3 GetTipOrientation()
        {
            //angle_3D tipOri;
            Vector3 o = new Vector3();
            //int returnValue = ArmGetTipOrientation(ref o);
//TODO: co zwraca zwykly?
			int returnValue = ArmGetTipOrientationUnitVector(ref o);

            if (returnValue == ARM_SUCCESS)
                return new Vector3(o.X, o.Y, o.Z);
            else
            {
                return new Vector3(0, 0, 0);
            }
        }


        internal bool IsButtonLeft()
        {
            uint buts = 0;
            int returnValue = ArmGetButtonsState(ref buts);

            if (returnValue == ARM_SUCCESS)
                return ((buts & 0x02) != 0);// button 1
            else
            {
                return false;
            }
        }

        internal bool IsButtonRight()
        {
            uint buts = 0;
            int returnValue = ArmGetButtonsState(ref buts);

            if (returnValue == ARM_SUCCESS)
                return ((buts & 0x01) != 0);// button 2
            else
            {
                return false;
            }
        }
    }
}
