﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Rotplucs
{
    static class Geometry
    {
        static PolyShape pS;

        public static void Init(PolyShape polyShape)
        {
            pS = polyShape;
        }

        /// <summary>
        /// dodaje wierzcholek we wskazanym punkcie i zwracaj jego indeks
        /// </summary>
        public static int AddVertax(Vector3 punkt)
        {
            return pS.AddVertex(punkt);
        }

        /// <summary>
        /// usuwa wszystkie trojkaty, ktore korzystaly z tego wierzchołka
        /// </summary>
        /// <param name="wierzcholek"></param>
        public static void RemoveVertax(int wierzcholek)
        {
            for (int j = 0; j < pS.indicesCount(); j += 3)
            {
                for (int k = 0; k < 3; k++)
                {
                    if (pS.GetIndices(j+k) == wierzcholek)   //sprawdz czy trójkąt ma wierzchołek który ma zostać usunięty
                    {
                        RemoveTriagle(j);
                        j -= 3;
                        break;
                    }
                }
            }

            //nie mozna usunac wierzcholka, bo wszystkie trojkaty sie zepsuja
            //vertices.Remove(vertices[wierzcholek]);
            //wierzcholkiUsuniete++;
        }

        /// <summary>
        /// dodaje nowy trojkat
        /// </summary>
        public static void AddTriangle(int numerWielokata, int p1, int p2, int p3, bool zapis = true)
        {
            if (zapis)
                EdycjaZmian.ZapiszDodanie(new Trojkat(p1, p2, p3, pS.indicesCount(), numerWielokata));

            pS.AddIndices(p1);
            pS.AddIndices(p2);
            pS.AddIndices(p3);

            pS.AddPolygons(numerWielokata);
        }


        public static void AddTriangle(int numerWielokata, Trojkat t)
        {
            Geometry.AddTriangle(numerWielokata, t.P1, t.P2, t.P3);
        }

        public static void AddTriangleAt( int numerWielokata, int p1, int p2, int p3, int start)
        {
            pS.InserIndices(start, p1);
            pS.InserIndices(start + 1, p2);
            pS.InserIndices(start + 2, p3);

            pS.InsertPolygons(start / 3, numerWielokata);
            //TODO:
        }

        /// <summary>
        /// usuwa cały trójkąt
        /// </summary>
        /// <param name="p"></param>
        public static void RemoveTriagle(int p, bool zapis = true)
        {
            if (zapis)
                EdycjaZmian.ZapiszUsuniecie(new Trojkat(pS.GetIndices(p), pS.GetIndices(p + 1), pS.GetIndices(p + 2), p, pS.GetPolygons(p / 3)));

            pS.RemoveIndicesAt(p+2);
            pS.RemoveIndicesAt(p + 1);
            pS.RemoveIndicesAt(p);

            pS.RemovePolygonsAt(p/3);
        }

        /// <summary>
        /// dla podanych punktów, wyznacza prostą i znajduje miejsce jej przecięcia z płaszczyzną
        /// </summary>
        /// <returns>Punkt przeciecia</returns>
        public static Vector3 WhereLineIntersectsSurface(Vector3 punkt1, Vector3 punkt2, Plaszczyzna p)
        {
            Vector3 wektorProstej = punkt1 - punkt2;

            //return wierzcholek2 + wektorProstej/2;
            float sumaWspolczynnikowPrzyT = Vector3.Dot(wektorProstej, p.Cross);
            float wspolczynnikT = (Vector3.Dot(p.Cross, p.Point) - Vector3.Dot(p.Cross, punkt2)) / sumaWspolczynnikowPrzyT;

            Vector3 punktPrzeciecia = wektorProstej * wspolczynnikT + punkt2;

            return punktPrzeciecia;
        }

        



        public static Vector3 WhereLineIntersectTriangle(Vector3 p1, Vector3 p2, int j)
        {
            return WhereLineIntersectTriangle(p1, p2, new Trojkat(pS.Triangles(j), pS.Triangles(j + 1), pS.Triangles(j + 2)));
        }

        public static Vector3 WhereLineIntersectTriangle(Vector3 p1, Vector3 p2, Trojkat t)
        {
            //czy to nie jest rzutowanie?
            Vector3 p0_pos = pS.VerticesPos(t.P1);
            Vector3 p1_pos = pS.VerticesPos(t.P2);
            Vector3 p2_pos = pS.VerticesPos(t.P3);

            Vector3 normalna = Normalna(p0_pos, p1_pos, p2_pos);

            Vector3 p = WhereLineIntersectsSurface(p1, p2, new Plaszczyzna(normalna, p0_pos));

            if (IsPointInTriangle(p, t))
                return p;
            return Vector3.Zero;
        }

        internal static void IsPointInWielokat(Vector3 punkt, List<int> list)
        {
            throw new NotImplementedException();
        }

#region TriangleTest
        public static bool IsPointInTriangle(Vector3 p, Trojkat t)
        {
            Vector3 p0_pos = pS.VerticesPos(t.P1);
            Vector3 p1_pos = pS.VerticesPos(t.P2);
            Vector3 p2_pos = pS.VerticesPos(t.P3);

            return IsPointInTriangle(p, p0_pos, p1_pos, p2_pos);
        }


        public static bool IsPointInTriangle(Vector3 p, Vector3 p0_pos, Vector3 p1_pos, Vector3 p2_pos)
        {
            return (SameSide(p, p0_pos, p1_pos, p2_pos) &&
               SameSide(p, p1_pos, p0_pos, p2_pos) &&
               SameSide(p, p2_pos, p0_pos, p1_pos));
        }

        private static bool SameSide(Vector3 p, Vector3 p0_pos, Vector3 p1_pos, Vector3 p2_pos)
        {
            Vector3 v1 = Normalna(p2_pos, p1_pos, p);
            Vector3 v2 = Normalna(p2_pos, p1_pos, p0_pos);

            return SameSide(v1, v2);
        }

        public static bool SameSide(Vector3 v1, Vector3 v2)
        {
            if (Vector3.Dot(v1, v2) >= 0)
                return true;
            else
                return false;
        }
#endregion

        public static Vector3 Normalna(Vector3 p0, Vector3 p1, Vector3 p2)
        {
            Vector3 v1 = p1 - p0;
            Vector3 v2 = p2 - p0;
            return Vector3.Normalize(Vector3.Cross(v1, v2));
        }

        public static Vector3 Normalna(int p1, int p2, int p3)
        {
            return Normalna(pS.VerticesPos(p1), pS.VerticesPos(p2), pS.VerticesPos(p3));
        }

        //zwraca trojkat w zaeżności od lewoskrętności i prawoskrętnosci
        internal static Trojkat GetTriangleUsingCross(int p1, int p2, int p3, Vector3 _cross)
        {
            if (Vector3.Dot(Normalna(p1, p2, p3), _cross) > 0)
                return new Trojkat(p1, p2, p3);
            else
                return new Trojkat(p3, p2, p1);
        }

        internal static void AddTriangleCross(int numerWielokata, int p1, int p2, int p3, Vector3 _cross)
        {
            Trojkat t = GetTriangleUsingCross(p1, p2, p3, _cross);
            Geometry.AddTriangle(numerWielokata, t);
        }


        public static Vector3 Rzutowanie(int vertex, Plaszczyzna p)
        {
            Vector3 punkt = pS.VerticesPos(vertex);
            return Rzutowanie(punkt, p);
        }

        public static Vector3 Rzutowanie(Vector3 punkt, Plaszczyzna p)
        {
            float odleglosc = OdlegloscPktDoPla(punkt, p);
            Vector3 przesuniecie = p.Cross * odleglosc;
            return punkt - przesuniecie;
        }

        //odległosc prawdziwa, a nie bezwzględna!
        public static float OdlegloscPktDoPla(Vector3 punkt, Plaszczyzna p)
        {
            return Vector3.Dot((punkt - p.Point), p.Cross);
        }

        /// <summary>
        /// każde cięcie płaszczyzną przez trójką powoduje jego podział na 2 części,
        /// jedna z jednym wierzchołkiem początkowym, a druga z dwoma
        /// </summary>
        /// <param name="trojkat">index początku trójkąta </param>
        /// <returns>
        /// zwraca który wierzchołek (0,1,2) z trójkąta jest pojedynczy
        /// -1 w przypadku, gdy, nie było cięcia
        /// </returns>
        public static int ZnajdzWierzcholekPojedynczy(int trojkat, Plaszczyzna p)
        {
            float wynik0 = Vector3.Dot((pS.VerticesPos(pS.Triangles(trojkat)) - p.Point), p.Cross);
            float wynik1 = Vector3.Dot((pS.VerticesPos(pS.Triangles(trojkat + 1)) - p.Point), p.Cross);
            float wynik2 = Vector3.Dot((pS.VerticesPos(pS.Triangles(trojkat + 2)) - p.Point), p.Cross);

            if ((wynik0 * wynik1) > 0 && (wynik0 * wynik2) < 0) //0 i 1 po tej samej stronie, a 2 po innej
                return 2;
            if ((wynik0 * wynik2) > 0 && (wynik0 * wynik1) < 0) //0 i 2 po tej samej stronie, a 1 po innej
                return 1;
            if ((wynik1 * wynik2) > 0 && (wynik1 * wynik0) < 0) //1 i 2 po tej samej stronie, a 0 po innej
                return 0;

            return -1;
        }

        public static float SignedAngle(Vector3 wektor1, Vector3 wektor2, Plaszczyzna p)
        {
            return (float)Math.Atan2(Vector3.Dot(Vector3.Cross(wektor1, wektor2), p.Cross), Vector3.Dot(wektor1, wektor2));
        }

        internal static float OdlegloscPomiedzy(int p1, int p2)
        {
            Vector3 p0_pos = pS.VerticesPos(p1);
            Vector3 p1_pos = pS.VerticesPos(p2);

            return (p0_pos - p1_pos).Length();
        }

        internal static Vector3[] CreateCone(Vector3 pos, Vector3 normalna, int kat, int wysokosc)
        {

            int wielkosc = (int)(wysokosc / Math.Tan(kat / 180.0 * Math.PI));
            //wektory prostopadłe, do wyznaczenia wierzchołków podstawy piramidki
            Vector3 prostopadly1 = new Vector3(normalna.Z - normalna.Y, normalna.X - normalna.Z, normalna.Y - normalna.X);
            prostopadly1.Normalize();

            Vector3 prostopadly2 = -prostopadly1;
            Vector3 prostopadly3 = Vector3.Cross(normalna, prostopadly1);
            Vector3 prostopadly4 = Vector3.Cross(normalna, prostopadly2);

            Vector3[] piramidka = new Vector3[5];
            piramidka[0] = pos + prostopadly1 * wielkosc;
            piramidka[1] = pos + prostopadly2 * wielkosc; //celowo wpisane 2 i 1 odwrotnie
            piramidka[2] = pos + prostopadly3 * wielkosc;
            piramidka[3] = pos + prostopadly4 * wielkosc;
            piramidka[4] = pos + normalna * wysokosc;

            return piramidka;
        }
    }
}
