﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace Rotplucs
{
    public static class Helper
    {
        public static bool Equals(float v1, float v2, float epsilon)
        {
            return (Math.Abs(v1 - v2) < epsilon);
        }

        public static bool Equals(Vector3 v1, Vector3 v2, float epsilon)
        {
            return (Equals(v1.X, v2.X, epsilon) &&
                    Equals(v1.Y, v2.Y, epsilon) &&
                    Equals(v1.Z, v2.Z, epsilon));

        }

        public static bool EqualsTo(this Vector3 v1, Vector3 v2)
        {
            return (Equals(v1, v2, 0.001F));
        }


        public static Vector3 ToVector(this Quaternion quat)
        {
            double sqx = quat.X * quat.X;
            double sqy = quat.Y * quat.Y;
            double sqz = quat.Z * quat.Z;

            var vector = new Vector3();
            vector.X = (float)Math.Atan2(2 * (quat.Z * quat.Y + quat.X * quat.W), 1 - 2 * (sqx + sqy));
            vector.Y = (float)Math.Asin(-2 * (quat.X * quat.Z - quat.Y * quat.W));
            vector.Z = (float)Math.Atan2(2 * (quat.X * quat.Y + quat.Z * quat.W), 1 - 2 * (sqy + sqz));

            return vector;
        }

        /// <summary>
        /// get rotation Quaternion from Matrix
        /// </summary>
        /// <param name="inMat"></param>
        /// <returns></returns>
        public static Quaternion DecomposeMatRot(this Matrix inMat)
        {
            Quaternion rot;
            Vector3 scale;
            Vector3 trans;
            inMat.Decompose(out scale, out rot, out trans);

            return rot;
        }

            
        public static Matrix CreateFromYawPitchRoll(Vector3 YPR)
        {
            return Matrix.CreateFromYawPitchRoll(YPR.X, YPR.Y, YPR.Z);
        }

        public static float ToDegree(this float radian)
        {
            return (float)(radian * 180 / Math.PI);
        }

        public static float ToRadian(this float degree)
        {
            return (float)(degree / 180.0F * Math.PI);
        }
    }
}