﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

namespace Rotplucs.Input
{
    public static class MyInput
    {
        public static _3DxConnection _3Dx;
        public static _MS_Connection _MS;
        public static _CS2Connection _CS2;
        public static KeyboardState _Keyboard;
        static MouseState _Mouse;

        public static void SetState(int x, int y, ButtonState left, ButtonState right)
        {
            x = (int)(x * Constants.WindowResolution.X / (double)Constants.WindowSize.X);
            y = (int)(y * Constants.WindowResolution.Y / (double)Constants.WindowSize.Y);

            _Mouse = new MouseState(x, y, 0, left, ButtonState.Released, right, ButtonState.Released, ButtonState.Released);
        }

        public static void Load(IntPtr windowHanlde)
        {
            //3D_mouse - asynchroniczny update, sam ustawia parametry w klasie _3Dx
            _3Dx = new _3DxConnection();
            if (!_3Dx.Start3Dx(windowHanlde))
                _3Dx = null;

            //joystick - synchroniczny update
            _CS2 = new _CS2Connection();
            if (!_CS2.StartCS2())
                _CS2 = null;

            //MicroScribe - brak updatu - odpytywanie urządzenia z poszczególnych ustawień
            _MS = new _MS_Connection();
            if (!_MS.StartMS(windowHanlde))
                _MS = null;

            CameraInput.Reset();
        }

        public static void Update()
        {
            _Keyboard = Keyboard.GetState();
            //_Mouse = Mouse.GetState();
            if (_CS2 != null)
                _CS2.Update();      //synchroniczny update

            CameraInput.Update();
        }

        //zmienne publiczne ustawiane z Forma ;p
        public static bool isCofnij = false;
        public static bool isPowtorz = false;
        public static bool isZapis = false;
        public static bool isOdczyt = false;
        public static bool isReset = false;
        public static bool isNew = false;
        public static int ConeAngle;

        internal static bool IsCofnijEdycje()
        {
            if (isCofnij)
            {
                isCofnij = false;
                return true;
            }
            
            return _Keyboard.IsKeyDown(Keys.Back);
        }

        internal static bool IsPowtorzEdycje()
        {
            if (isPowtorz)
            {
                isPowtorz = false;
                return true;
            }
            return _Keyboard.IsKeyDown(Keys.Enter);
        }

        internal static bool IsZapisRzezbe()
        {
            if (isZapis)
            {
                isZapis = false;
                return true;
            }
            return _Keyboard.IsKeyDown(Keys.Space);
        }

        internal static bool IsOdczytRzezbe()
        {
            if (isOdczyt)
            {
                isOdczyt = false;
                return true;
            }
            return _Keyboard.IsKeyDown(Keys.O);
        }

        internal static bool IsResetWidoku()
        {
            if (isReset)
            {
                isReset = false;
                return true;
            }

            if(MyInput._3Dx != null && MyInput._3Dx.Reset)
                return true;
            if (MyInput._CS2 != null && MyInput._CS2.Reset)
                return true;
            return _Keyboard.IsKeyDown(Keys.R);
        }

        internal static bool IsNewPolyShape()
        {
            if (isNew)
            {
                isNew = false;
                return true;
            }

            return false;
        }




        internal static bool IsCutingStart()
        {
            if (_MS != null)
            {
                if (_MS.GetLeftButton())
                    return true;
                else
                    return false;
            }
			
            return _Mouse.LeftButton == ButtonState.Pressed;
        }

        internal static bool IsHitingStart()
        {
            if (_MS != null)
            {
                if (_MS.GetRightButton())
                    return true;
                else
                    return false;
            }

            return _Mouse.RightButton == ButtonState.Pressed;
        }



        internal static Point GetCursorPos()
        {
            if (_MS != null)
                return _MS.GetPosition2D();

            return new Point(_Mouse.X, _Mouse.Y);
        }

        internal static Vector3 Get3DPos()
        {
            if (_MS != null)
                return _MS.GetPosition3D();

            float x = (_Mouse.X - Constants.WindowResolution.X / 2);
            float y = (Constants.WindowResolution.Y / 2 - _Mouse.Y);
            return CameraInput.Poprawka(new Vector3(x, y, 0));
        }

        internal static Vector3 Get3DRot()
        {
            Vector3 orient;

            if (_MS != null)
            {
                orient = _MS.GetRotation3D();
                orient.X = -orient.X;
            }
            else
                orient = CameraInput.rotationVector();//(Get3DPos() - OtherInput.Camera)/100;

            orient.Normalize();

            return orient;
        }
    }
}
