using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Rotplucs.Input;

namespace Rotplucs
{
    public class SculptorGame : SimpleXNAGame
    {
        public SculptorGame() :base() { }
        public SculptorGame(IntPtr drawSurface) : base(drawSurface) { }

#region Load

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            base.LoadContent();

            PolyShape = new PolyShape(GraphicsDevice, Constants.PolyShapeSize, Constants.PolyShapePos);

            TextOnScreen.Load(LoadText("Courier New"), GraphicsDevice);
            LineMove.Load(GraphicsDevice);
            MouseCursor.Load(LoadGraphic("kursor"), GraphicsDevice);
            LineCone.Load(GraphicsDevice);

            PolyShape.Load(PerlinNoise.Generate(GraphicsDevice), LoadShader("SimpleColor"));
        }

#endregion


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            OtherInput.Update(PolyShape);

            //rysowa na podstawie Microscriba
            LineCone.Update(MyInput.Get3DPos(), MyInput.Get3DRot());

            base.Update(gameTime);
        }



        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.DimGray);
            GraphicsDevice.BlendState = BlendState.Opaque;
            GraphicsDevice.DepthStencilState = DepthStencilState.Default;

            if (Error.Exit)
                return;

            PolyShape.Draw(_world, _view, _projection);
            LineMove.Draw(_world, _view, _projection);
            LineCone.Draw(_world, _view, _projection);

            MouseCursor.Draw();
            TextOnScreen.Draw(PolyShape.TrianglesCount, PolyShape.VerticesCount, PolyShape.PolygonCount);
            TextOnScreen.DrawDebug();

            base.Draw(gameTime);
        }

        private PolyShape PolyShape { get; set; }
    }
}
