﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

using Rotplucs.Input;

namespace Rotplucs
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class SimpleXNAGame : Microsoft.Xna.Framework.Game
    {
        protected readonly GraphicsDeviceManager _graphics;
        protected Matrix _world, _view, _projection;

        private IntPtr drawSurface;
        public SimpleXNAGame(IntPtr drawSurface)
        {
            _graphics = new GraphicsDeviceManager(this)
            {
                PreferMultiSampling = true      //antialiasing
            };

            Content.RootDirectory = "Content";
            this.drawSurface = drawSurface;
            _graphics.PreparingDeviceSettings += new EventHandler<PreparingDeviceSettingsEventArgs>(graphics_PreparingDeviceSettings);
            System.Windows.Forms.Control.FromHandle((this.Window.Handle)).VisibleChanged +=  new EventHandler(Game1_VisibleChanged);           
        }

        void graphics_PreparingDeviceSettings(object sender, PreparingDeviceSettingsEventArgs e)
        {
            e.GraphicsDeviceInformation.PresentationParameters.DeviceWindowHandle = drawSurface;
        }

        private void Game1_VisibleChanged(object sender, EventArgs e)
        {
                if (System.Windows.Forms.Control.FromHandle((this.Window.Handle)).Visible == true)
                    System.Windows.Forms.Control.FromHandle((this.Window.Handle)).Visible = false;
        }





        public SimpleXNAGame()
        {
            _graphics = new GraphicsDeviceManager(this)
            {
                //IsFullScreen = true,
                PreferMultiSampling = true      //antialiasing
            };
            Content.RootDirectory = "Content";
        }

#region Initialize

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            InitializeResolutionByWidth(Constants.WindowResolution.X);

            base.Initialize();
        }

        /// <summary>
        /// ustawia rozdizelczość w zaleznosci od AspectRatio
        /// </summary>
        private void InitializeResolutionByWidth(int width)
        {
            float aspectRatio = GraphicsDevice.DisplayMode.AspectRatio;
            int height = (int)Math.Round(width / aspectRatio);
            _graphics.PreferredBackBufferWidth = width;
            _graphics.PreferredBackBufferHeight = height;
            _graphics.ApplyChanges();
			//
            Constants.WindowResolution.Y = height;
        }

#endregion
#region Load

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            _world = Matrix.Identity;
            _view = Matrix.CreateLookAt(CameraInput.cameraPos, GetLookAt(), Vector3.Up);
            _projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4, GraphicsDevice.Viewport.AspectRatio, 1, 10000);

            MyInput.Load(Window.Handle);
        }

        private Vector3 GetLookAt()
        {
            Vector3 lookAt = CameraInput.cameraPos;
            lookAt.Z -= 5;
            return lookAt;
        }



        internal Texture2D LoadTexture(string fileName)
        {
            return Content.Load<Texture2D>("Textures\\" + fileName);
        }

        internal Texture2D LoadGraphic(string fileName)
        {
            return Content.Load<Texture2D>("Graphics\\" + fileName);
        }

        internal Effect LoadShader(string fileName)
        {
            return Content.Load<Effect>(fileName);
        }

        internal SpriteFont LoadText(string fontName)
        {
            return Content.Load<SpriteFont>(fontName);
        }

#endregion

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            MyInput.Update();
            _view = Matrix.CreateLookAt(CameraInput.cameraPos, GetLookAt(), Vector3.Up);

            _world = CameraInput.worldDifineByCamera;
            //to mnożemy macierz o każdy drobny ruch, dzięki czemu zależny jest on od poprzedniej pozycj - to chcielisy uzyskać
            //np Roll jest zawsze względem aktualnej osi patrzenia

            base.Update(gameTime);
        }


        bool fDown = false;

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            if (MyInput._Keyboard.IsKeyDown(Keys.F))
            {   
                if(!fDown)
                {
                    _graphics.ToggleFullScreen();


                    fDown = true;
                }
            }
            else
                fDown = false;



            base.Draw(gameTime);
        }
    }
}
