﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace Rotplucs
{
    public class Plaszczyzna
    {
        public Vector3 Cross { get; private set; }
        public Vector3 Point { get; private set; }

        public Plaszczyzna(Vector3 _cross, Vector3 _point)
        {
            Cross = Vector3.Normalize(_cross);
            Point = _point;
        }

        public Plaszczyzna(Vector3 p1, Vector3 p2, Vector3 p3)
            : this(Geometry.Normalna(p1, p2, p3), p1)
        {

        }

        public Plaszczyzna(Vector3 p1, Vector3 p2, Vector3 p3, Vector3 insidePoint)
            : this(Geometry.Normalna(p1, p2, p3), p1)
        {
            SetPunktWnetrza(insidePoint);
        }


        public void SetPunktWnetrza(Vector3 vector)
        {
            Cross = (Geometry.OdlegloscPktDoPla(vector, this) < 0) ? -Cross : Cross;
        }
    }

    class PunktNaKrawedzi
    {
        public int V1 { get; private set; }
        public int V2 { get; private set; }
        public int Point { get; private set; }

        public PunktNaKrawedzi(int v1, int v2, int p)
        {
            V1 = v1;
            V2 = v2;
            Point = p;
        }
    }

    public class Wielokat
    {
        public Plaszczyzna Plaszcz { get; private set; }
        //zmienic na private set!
        public List<Krawedz> Krawedzie { get; set; }

        public Wielokat(Plaszczyzna p)
        {
            Plaszcz = p;
            Krawedzie = new List<Krawedz>();
        }
    }
    
    public class Krawedz
    {
        public Krawedz(int p1, int p2)
        {
            P1 = p1;
            P2 = p2;
        }

        public int P1 { get; set; }
        public int P2 { get; set; }

        public override string ToString()
        {
            return P1 + ", " + P2;
        }

        public override bool Equals(object obj)
        {
            var k = obj as Krawedz;
            if (k != null)
            {
                return (this.P1 == k.P1 && this.P2 == k.P2);
            }
            return base.Equals(obj);
        }

        //najpierw szuka po haszkodzie, a potem chyba porównuje z equalsem
        public override int GetHashCode()
        {
            if (P1 == null) return 0;
            if (P2 == null) return 0;
            return P1.GetHashCode() + P2.GetHashCode();
        }
    }

    public class Trojkat
    {
        public int NumerWielokata;
        public Trojkat(int p1, int p2, int p3)
        {
            P1 = p1;
            P2 = p2;
            P3 = p3;
            Pozycja = 0;
        }

        public Trojkat(int p1, int p2, int p3, int p, int numerWielokata)
        {
            P1 = p1;
            P2 = p2;
            P3 = p3;
            Pozycja = p;
            NumerWielokata = numerWielokata;
        }
        public int P1 { get; set; }
        public int P2 { get; set; }
        public int P3 { get; set; }

        public int Pozycja { get; set; }

        public override string ToString()
        {
            return P1 + ", " + P2 + ", " + P3 + "  -  " + Pozycja;
        }

        public Vector3 Normalna()
        {
            return Geometry.Normalna(P1, P2, P3);
        }
    }


    
}
