﻿using System;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using System.Collections.Generic;

namespace Rotplucs
{
    public class PolyShape
    {
        private int wierzcholkiUsuniete = 0;
        
        //konstruktor
        private readonly Vector3 _size;
        private readonly Vector3 _pos; 
        private readonly GraphicsDevice _gd;
        
        //load
        private Effect _simpleColorEffect;
        private Texture2D _crateTexture;
        private Texture2D _crateGrad;
        private Texture2D _cratePerm;

        private VertexBuffer _verticesBuffer;
        private IndexBuffer _indicesBuffer;
        private List<VertexPositionNormalTexture> vertices;
        private List<UInt16> indices;
        private List<int> Polygons;

        //musi być słownik bo inaczej to przymula...
        private HashSet<Krawedz> InvisibleEdges;
        private int NumerWirtualnegoWierzcholka = -1;   //zaczynamy od -2, bo gdzieś wykorzystujemy -1 ;D
        private UInt16[] I;

        public PolyShape(GraphicsDevice gd, Vector3 size, Vector3 pos)
        {
            _size = size;
            _pos = pos;
            _gd = gd;
            Geometry.Init(this);

            StartingCube.SetPosAndSize(_pos, _size);
        }

        /// <summary>
        /// ustawia tekstury i tworzy początkową kostkę
        /// </summary>
        public void Load(Noise n, Effect shader)
        {
            _simpleColorEffect = shader;
            _crateTexture = n.texture;
            _crateGrad = n.grad;
            _cratePerm = n.perm;

            _simpleColorEffect.Parameters["tex"].SetValue(_crateTexture);
            _simpleColorEffect.Parameters["gra"].SetValue(_crateGrad);
            _simpleColorEffect.Parameters["per"].SetValue(_cratePerm);

            LoadCube();
        }

        internal void Clear()
        {
            vertices = new List<VertexPositionNormalTexture>();
            indices = new List<UInt16>();
            Polygons = new List<int>();
            InvisibleEdges = new HashSet<Krawedz>();
            NumerWirtualnegoWierzcholka = -1;

            EdycjaZmian.EdycjaStart();
        }

        /// <summary>
        /// tworzy początkową kostkę
        /// </summary>
        public void LoadCube()
        {
            Clear();

            vertices.AddRange(StartingCube.CreateVertexBuffer());
            indices.AddRange(StartingCube.CreateIndexBuffer());
            Polygons.AddRange(StartingCube.CreateCubePolygonList());

            var listaKrawedzi = StartingCube.DodajInvisible();
            foreach (var k in listaKrawedzi)
            {
                InvisibleEdges.Add(k);
            }

            polygonCount = StartingCube.GetPolygonCount();

            SetDataToBuffer();
        }


#region invisible
        public int polygonCount;

        public int GetNewPolygon()
        {
            return ++polygonCount;
        }

        public bool IsVisibleEdge(int p1, int p2)
        {
            if (p1 > p2)
            {
                int tmp = p1;
                p1 = p2;
                p2 = tmp;
            }
            if (InvisibleEdges.Contains(new Krawedz(p1, p2)))
                return false;
            else
                return true;
        }

        public void AddInvisible(int p1, int p2)
        {
            if (p1 > p2)
            {
                int tmp = p1;
                p1 = p2;
                p2 = tmp;
            }
            if (!InvisibleEdges.Contains(new Krawedz(p1, p2)))
                InvisibleEdges.Add(new Krawedz(p1, p2));
        }

        internal void AddInvisible(Krawedz k)
        {
            AddInvisible(k.P1, k.P2);
        }

        public int GetNewVirtualVeraxNumber()
        {
            return --NumerWirtualnegoWierzcholka;
        }


#endregion ivisible


        /// <summary>
        /// ustawia bufory do renderowania
        /// </summary>
        private void SetDataToBuffer()
        {
            VertexPositionNormalTexture[] V = GenerateNormals();    //vertices.ToArray()

            if (V.Length == 0)
            {
                Error.NoVertices();
                return;
            }


            _verticesBuffer = new VertexBuffer(_gd, VertexPositionNormalTexture.VertexDeclaration, V.Length, BufferUsage.WriteOnly);
            _verticesBuffer.SetData(V);
            _indicesBuffer = new IndexBuffer(_gd, IndexElementSize.SixteenBits, indices.Count, BufferUsage.WriteOnly);
            _indicesBuffer.SetData(I);
        }

        //każdy trójkąt ma 3 własne wierzchołki z nromalnymi skierowanym tak jak trójkąt.
        private VertexPositionNormalTexture[] GenerateNormals()
        {
            VertexPositionNormalTexture[] V = new VertexPositionNormalTexture[indices.Count];
            I = new UInt16[indices.Count];

            for (int i = 0; i < indices.Count / 3; i++)
            {
                Vector3 firstvec = vertices[indices[i * 3 + 1]].Position - vertices[indices[i * 3]].Position;
                Vector3 secondvec = vertices[indices[i * 3]].Position - vertices[indices[i * 3 + 2]].Position;
                Vector3 normal = Vector3.Cross(firstvec, secondvec);
                normal.Normalize();
                V[i * 3].Normal = normal;
                V[i * 3 + 1].Normal = normal;
                V[i * 3 + 2].Normal = normal;

                V[i * 3    ].Position = vertices[indices[i * 3]].Position;
                V[i * 3 + 1].Position = vertices[indices[i * 3 + 1]].Position;
                V[i * 3 + 2].Position = vertices[indices[i * 3 + 2]].Position;

                I[i * 3] = (ushort)(i * 3);
                I[i * 3 + 1] = (ushort)(i * 3 + 1);
                I[i * 3 + 2] = (ushort)(i * 3 + 2);
            }



            return V;
        }

        /// <summary>
        /// rysuje cala rzeźbe
        /// </summary>
        public void Draw(Matrix world, Matrix view, Matrix projection)
        {
            _gd.SetVertexBuffer(_verticesBuffer);
            _gd.Indices = _indicesBuffer;
            _simpleColorEffect.Parameters["WVP"].SetValue(world * view * projection);

            Vector3 diffuseLightDirection = new Vector3(0, 0, -1);
            diffuseLightDirection = Vector3.Transform(diffuseLightDirection,Matrix.Invert(world));

            _simpleColorEffect.Parameters["DiffuseDirection"].SetValue(diffuseLightDirection);

            foreach (EffectPass pass in _simpleColorEffect.CurrentTechnique.Passes)
            {
                pass.Apply();
                _gd.DrawIndexedPrimitives(PrimitiveType.TriangleList, 0, 0, vertices.Count, 0, indices.Count / 3);
            }
        }




        /*
        public void SetCuttingPlane(Vector3 punkt1, Vector3 punkt2, Vector3 punkt3)
        {
            Geometry.AddTriangle(vertices.Count    , vertices.Count +1, vertices.Count + 2);
            Geometry.AddTriangle(vertices.Count + 2, vertices.Count + 1, vertices.Count);

            Geometry.AddVertax(punkt1);
            Geometry.AddVertax(punkt2);
            Geometry.AddVertax(punkt3);

            SetDataToBuffer();
        }*/



        public void CutSurface(Vector3 firstClick, Vector3 secondClick, Vector3 point)
        {
            EdycjaZmian.NowaEdycja();
            Cut.MakeACut(this, firstClick, secondClick, point);
            SetDataToBuffer();
        }

        public void HitSurface(Vector3 from, Vector3 to)
        {
            EdycjaZmian.NowaEdycja();
            Hit.HitSurface(this, from, to);
            SetDataToBuffer();
        }

        /*
        public void SmoothSurface(Vector3 from, Vector3 to)
        {
            Smooth.SmoothSurface(this, from, to);
            SetDataToBuffer();
        }
         */

        public void CofnijEdycje()
        {
            EdycjaZmian.CofnijEdycje();
            SetDataToBuffer();
        }

        public void PowtorzEdycje()
        {
            EdycjaZmian.PowtorzEdycje();
            SetDataToBuffer();
        }

        public int getPolyNumberFromTriangle(int i)
        {
            return Polygons[i / 3];
        }

        public int Triangles(int i)
        {
            return indices[i];
        }
        public Vector3 VerticesPos(int i)
        {
            return vertices[i].Position;
        }

//Info
        public int TrianglesCount { get { return indices.Count / 3; } }
        public int VerticesCount { get { return vertices.Count - wierzcholkiUsuniete; } }
        public int PolygonCount { get { return polygonCount; } }
//_Info

        internal int AddVertex(Vector3 punkt)
        {
            VertexPositionNormalTexture wierzcholek = new VertexPositionNormalTexture();
            wierzcholek.Position = punkt;

            vertices.Add(wierzcholek);
            return vertices.Count - 1;
        }

        internal int verticesCount()
        {
            return vertices.Count;
        }


    //indices Access
        internal int indicesCount()
        {
            return indices.Count;
        }

        internal void RemoveIndicesAt(int p)
        {
            indices.RemoveAt(p);
        }

        internal void AddIndices(int p)
        {
            indices.Add((ushort)(p));
        }

        internal void InserIndices(int start, int p)
        {
            indices.Insert(start, (ushort)(p));
        }

        internal int GetIndices(int p)
        {
            return indices[p];
        }


    //polygon access
        internal void AddPolygons(int numerWielokata)
        {
            Polygons.Add(numerWielokata);
        }

        internal void InsertPolygons(int p, int numerWielokata)
        {
            Polygons.Insert(p, numerWielokata);
        }

        internal void RemovePolygonsAt(int p)
        {
            Polygons.RemoveAt(p);
        }

        internal int GetPolygons(int p)
        {
            return Polygons[p];
        }

        internal List<ushort> GetAllIndices()
        {
            return indices;
        }

        internal List<VertexPositionNormalTexture> GetAllVertices()
        {
            return vertices;
        }


        internal void Save()
        {
            FileIO.Save(this);
            SetDataToBuffer();
        }

        internal void Open()
        {
            FileIO.Open(this);
            SetDataToBuffer();
        }
    }
}
