﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace Rotplucs
{
    public class Ray
    {
        public List<Krawedz> Krawedzie;
        public Dictionary<int, int> PunktyNaTrojkatach;

        public Ray(List<Krawedz> krawedzie, Dictionary<int, int> punktyNaTrojkatach)
        {
            Krawedzie = krawedzie;
            PunktyNaTrojkatach = punktyNaTrojkatach;
        }
    }

    public class RayTracedTriangle
    {
        public int NumerTrojkata;
        public Vector3 PunktPrzeciecia;
        public float Dlugosc;

        public RayTracedTriangle(int numerTrojkata, Vector3 punktPrzeciecia, float dlugosc)
        {
            NumerTrojkata = numerTrojkata;
            PunktPrzeciecia = punktPrzeciecia;
            Dlugosc = dlugosc;
        }


    }

    public static class RayTrace
    {
        public static RayTracedTriangle First(Vector3 from, Vector3 to, PolyShape pS)
        {
            return RayTriangles(from, to, pS).Find(x => x.Dlugosc > 0);

        }

        public static List<RayTracedTriangle> RayTriangles(Vector3 from, Vector3 to, PolyShape pS)
        {
            List<RayTracedTriangle> trojkatyPrzeswitlone = new List<RayTracedTriangle>();
            for (int i = 0; i < pS.indicesCount(); i += 3)
            {
                Vector3 punkt_pos = Geometry.WhereLineIntersectTriangle(from, to, i);
                if (punkt_pos != Vector3.Zero)
                {
                    float length = (punkt_pos - from).Length();
                    //dodajemy te zle, tez bo ray musi potem wiedziec cy jest w srodku
                    if (!Geometry.SameSide((to - from), (punkt_pos - from)))
                        length = -length;

                    trojkatyPrzeswitlone.Add(new RayTracedTriangle(i, punkt_pos, length));
                }
            }

            trojkatyPrzeswitlone.Sort((x, y) => x.Dlugosc.CompareTo(y.Dlugosc));

            return trojkatyPrzeswitlone;
        }

        public static Ray Trace(Vector3 from, Vector3 to, int startPoint, PolyShape pS)
        {
            int entryPoint = 0;
            List<Krawedz> krawedzie = new List<Krawedz>();
            Dictionary<int, int> punktyNaTrojkatach = new Dictionary<int, int>();
            List<RayTracedTriangle> trojkatyPrzeswitlone = RayTriangles(from, to, pS);

            if (trojkatyPrzeswitlone.Count == 0)
                return null;

            //usuwamy te przed promieniem
            bool IsInside = false;
            while (trojkatyPrzeswitlone[0].Dlugosc < 0)
            {
                trojkatyPrzeswitlone.RemoveAt(0);
                if (trojkatyPrzeswitlone.Count == 0)
                    return null;
                IsInside = !IsInside;
            }

            if (IsInside)
                entryPoint = startPoint;

            foreach (var t in trojkatyPrzeswitlone)
            {
                int punkt = Geometry.AddVertax(t.PunktPrzeciecia);
                punktyNaTrojkatach.Add(t.NumerTrojkata, punkt);

                if (IsInside)
                    krawedzie.Add(new Krawedz(entryPoint, punkt));
                else
                    entryPoint = punkt;

                IsInside = !IsInside;
            }

            //istotne punkty From i TO, jedna z krawęxdzi to jest od Froma
            //poprawić, ze krawędzie są dodawane dopiero po sprawdzeniu wszystkich trójkątów!

            return new Ray(krawedzie, punktyNaTrojkatach);
        }
    }
}
