float4x4 WVP;
texture2D tex;
texture2D gra;
texture2D per;
float3 DiffuseDirection;


float PI = 3.1415926535;

struct VertexShaderInput
{
    float4 Position : POSITION0;
	float4 Normal : NORMAL0;
};

struct VertexShaderOutput
{
    float4 Position : POSITION0;
	float4 Pos3D	: COLOR0;
	float4 Normal : NORMAL0;
};

sampler2D texSampler = sampler_state
{
    Texture = <tex>;
	MinFilter = POINT;
    MagFilter = POINT;
    MipFilter = NONE;
	AddressU = WRAP;
	AddressV = WRAP;
	MaxAnisotropy = 16;
};

sampler2D permSampler = sampler_state
{
    Texture = <per>;
	MinFilter = POINT;
    MagFilter = POINT;
    MipFilter = NONE;
	AddressU = WRAP;
	AddressV = WRAP;
	MaxAnisotropy = 16;
};

sampler1D gradSampler = sampler_state
{
    Texture = <gra>;
	MinFilter = POINT;
    MagFilter = POINT;
    MipFilter = NONE;
	AddressU = WRAP;
	AddressV = WRAP;
	MaxAnisotropy = 16;
};

VertexShaderOutput VertexShaderFunction(VertexShaderInput input)
{
    VertexShaderOutput output;
	output.Pos3D =  input.Position;
	output.Position = mul(input.Position, WVP);
	output.Normal = input.Normal;

    return output;
}




float toFloat(float4 f)
{
	float Out = f.a;
	Out*=256;
	Out += f.r;
	Out*=256;
	Out += f.g;
	Out*=256;
	Out += f.b;
	return Out;
}

float perm(float x)
{
  return toFloat(tex2D(permSampler, x ));
}

float perm(float2 x)
{
	return perm(x.x + perm(x.y)) / 256;
}



float grad(float x, float3 p)
{
	x*=3;
	float ax = toFloat(tex1D(gradSampler, x ) )-1;
	float ay = toFloat(tex1D(gradSampler, x+1)) -1;
	float az = toFloat(tex1D(gradSampler, x+2) )-1;
	return float3( ax, ay, az);
}


float3 fade(float3 t)
{
  return t * t * t * (t * (t * 6 - 15) + 10); // new curve
}

// 3D version

float inoise(float3 p)
{
  float3 P = fmod(floor(p), 256.0);
  p -= floor(p);
  float3 f = fade(p);

  // HASH COORDINATES FOR 6 OF THE 8 CUBE CORNERS

  float A = perm(P.x) + P.y;
  float AA = perm(A) + P.z;
  float AB = perm(A + 1) + P.z;
  float B =  perm(P.x + 1) + P.y;
  float BA = perm(B) + P.z;
  float BB = perm(B + 1) + P.z;

  // AND ADD BLENDED RESULTS FROM 8 CORNERS OF CUBE
  
  //return p.x + perm(   p.y + perm(   p.z   ) % 256  ) %16;

  return grad(   p.x + perm(   p.y + perm(   p.z   ) % 256  ) %16, p  );

  return lerp(
    lerp(lerp(grad(perm(AA), p),
              grad(perm(BA), p + float3(-1, 0, 0)), f.x),
         lerp(grad(perm(AB), p + float3(0, -1, 0)),
              grad(perm(BB), p + float3(-1, -1, 0)), f.x), f.y),
    lerp(lerp(grad(perm(AA + 1), p + float3(0, 0, -1)),
              grad(perm(BA + 1), p + float3(-1, 0, -1)), f.x),
         lerp(grad(perm(AB + 1), p + float3(0, -1, -1)),
              grad(perm(BB + 1), p + float3(-1, -1, -1)), f.x), f.y), f.z);
}


/*
float PerlinNoise(float3 pos){
	return inoise(pos);
}

float AbsNoise(float3 pos) {
    return abs(PerlinNoise(pos));
}

float FBMNoise(float3 pos) {
    float value = 0.0f;
    float frequency = InitialFrequency;
    float amplitude = 1.0f;
    // Classic FBM loop
    //for ( int i=0; i < Octaves; i++ )
    {
        float noiseValue = AbsNoise(pos);
        value += amplitude * noiseValue;
        frequency *= Lacunarity;
        amplitude *= Amplitude;
    }
    return value;
}



static float stripes(float x, float f) {
    float t = .5 + .5 * sin(f * 2*PI * x);
    return t * t - .5;
}

float MarbleNoise(float3 pos) { 
    return stripes(pos.x + 2 * FBMNoise(pos), 1.6f);
}
*/










#define ONE 0.00390625
#define ONEHALF 0.001953125

float fade(float t) {
  //return t*t*(3.0-2.0*t); // Old fade
  return t*t*t*(t*(t*6.0-15.0)+10.0); // Improved fade
}

 
float MyNoise(float3 P)
{
  float3 Pi = ONE*floor(P)+ONEHALF; 
                                 
  float3 Pf = P-floor(P);
  
  // Noise contributions from (x=0, y=0), z=0 and z=1
  float perm00 = perm(Pi.xy);
  float perm00z = perm(float2(perm00, Pi.z))%16;

  float3  grad000 = tex2D(texSampler, float2(perm00, Pi.z)).rgb * 4.0 - 1.0;
  float n000 =  dot(grad000, Pf);
  float3  grad001 = tex2D(texSampler, float2(perm00, Pi.z + ONE)).rgb * 4.0 - 1.0;
  float n001 = dot(grad001, Pf - float3(0.0, 0.0, 1.0));

  // Noise contributions from (x=0, y=1), z=0 and z=1
  float perm01 = perm( Pi.xy + float2(0.0, ONE)) ;
  float3  grad010 = tex2D(texSampler, float2(perm01, Pi.z)).rgb * 4.0 - 1.0;
  float n010 = dot(grad010, Pf - float3(0.0, 1.0, 0.0));
  float3  grad011 = tex2D(texSampler, float2(perm01, Pi.z + ONE)).rgb * 4.0 - 1.0;
  float n011 = dot(grad011, Pf - float3(0.0, 1.0, 1.0));

  // Noise contributions from (x=1, y=0), z=0 and z=1
  float perm10 = perm( Pi.xy + float2(ONE, 0.0));
  float3  grad100 = tex2D(texSampler, float2(perm10, Pi.z)).rgb * 4.0 - 1.0;
  float n100 = dot(grad100, Pf - float3(1.0, 0.0, 0.0));
  float3  grad101 = tex2D(texSampler, float2(perm10, Pi.z + ONE)).rgb * 4.0 - 1.0;
  float n101 = dot(grad101, Pf - float3(1.0, 0.0, 1.0));

  // Noise contributions from (x=1, y=1), z=0 and z=1
  float perm11 = perm( Pi.xy + float2(ONE, ONE)) ;
  float3  grad110 = tex2D(texSampler, float2(perm11, Pi.z)).rgb * 4.0 - 1.0;
  float n110 = dot(grad110, Pf - float3(1.0, 1.0, 0.0));
  float3  grad111 = tex2D(texSampler, float2(perm11, Pi.z + ONE)).rgb * 4.0 - 1.0;
  float n111 = dot(grad111, Pf - float3(1.0, 1.0, 1.0));

  // Blend contributions along x
  float4 n_x = lerp(float4(n000, n001, n010, n011), float4(n100, n101, n110, n111), fade(Pf.x));

  // Blend contributions along y
  float2 n_xy = lerp(n_x.xy, n_x.zw, fade(Pf.y));

  // Blend contributions along z
  float n_xyz = lerp(n_xy.x, n_xy.y, fade(Pf.z));
 
  return n_xyz;
}

float turbulence(int octaves, float3 P, float lacunarity, float gain)
{	
  float sum = 0;
  float scale = 1;
  float totalgain = 1;
  for(int i=0;i<octaves;i++){
    sum += totalgain*abs(MyNoise(P*scale));
    scale *= lacunarity;
    totalgain *= gain;
  }
  return sum;
}


float MarbleNoise(float3 pos) { 
	return sqrt(sqrt(abs(sin( pos.x/2 + 3 * turbulence(8, pos, 2, 0.5))-0.1) + 0.1));
}



float4 PixelShaderFunction(VertexShaderOutput input) : COLOR0
{
	float dzielnik = 390;
	float x = input.Pos3D[0]/dzielnik; 
	float y = input.Pos3D[1]/dzielnik;
	float z = input.Pos3D[2]/dzielnik;


	float ax = tex1D(gradSampler,0).g;
	float ay = tex1D(gradSampler,0.01).g;
	float az = tex1D(gradSampler,0.02).g;
	///float3 diffuse = float3( x, y, z);


	float col = MarbleNoise(float3(x,y,z));	//marmur
	//float col = abs( sin( x*20 + sin(y*20 + sin(z*20) )  ) );		//testowe

	
    float4 diffuseVector = saturate(dot(-DiffuseDirection,input.Normal));


	float3 diffuse = float3( col, col, col);
	return float4(diffuse,1) * (diffuseVector/4 + 0.75);
}



float4 PixelShaderWireframeFunction(VertexShaderOutput input) : COLOR0
{
	return  float4(0.5,0.5,0.5,1);
}


technique Technique1
{
	pass Pass1
    {
		FillMode = Solid;
        VertexShader = compile vs_3_0 VertexShaderFunction();
        PixelShader = compile ps_3_0 PixelShaderFunction();
    }

	pass Pass2
    {
		FillMode = Wireframe;
        VertexShader = compile vs_3_0 VertexShaderFunction();
        PixelShader = compile ps_3_0 PixelShaderWireframeFunction();
    }
}
